(* $Id: gradient.mli,v 1.7 2014/02/27 16:36:36 fclement Exp $ *)

open Ugraphics;;

type rectangle_gradient_mode =
   | Rect_Horizontal of Ugraphics.from_color * Ugraphics.to_color
      (** Horizontal gradient. *)
   | Rect_Vertical of from_color * to_color
      (** Vertical gradient. *)
   | Rect_Diagonal1 of from_color * to_color
      (** Diagonal gradient along the first bissector of the rectangle. *)
   | Rect_Diagonal2 of from_color * to_color
      (** Diagonal gradient along the second bissector of the rectangle. *)
   | Rect_Radial_Rectangular of
       from_color * to_color *
       Ugraphics.x * Ugraphics.y * Ugraphics.aspect_ratio
      (** Radial gradient with rectangular isocolor curves. *)
   | Rect_Radial_Elliptic of from_color * to_color * x * y * aspect_ratio
      (** Radial gradient with elliptic isocolor curves. *)
;;

(** Gradient modes to fill rectangles. Each mode specifies a gradient
    from [from_color] to [to_color], with the corresponding method.
    For constructors [Rect_Radial_Rectangular] and [Rect_Radial_Elliptic],
    the [x] and [y] parameters are the coordinates of the gradient center,
    and [ratio] measures the horizontal over vertical ratio for closed
    isocolor curves: width over height ratio for rectangular isocolor curves,
    and horizontal axis over vertical axis for elliptic isocolor curves.
    
    Raise [Failure] when the gradient center is outside the rectangle. *)

type ellipse_gradient_mode =
   | Ellipse_Horizontal of from_color * to_color
      (** Horizontal gradient. *)
   | Ellipse_Vertical of from_color * to_color
      (** Vertical gradient. *)
   | Ellipse_Diagonal1 of from_color * to_color
      (** Diagonal gradient along the first bissector of the circonscript
        rectangle of the ellipse. *)
   | Ellipse_Diagonal2 of from_color * to_color
      (** Diagonal gradient along the second bissector of the circonscript
        rectangle of the ellipse. *)
   | Ellipse_Radial_Rectangular of from_color * to_color * x * y * aspect_ratio
      (** Radial gradient with rectangular isocolor curves. *)
   | Ellipse_Radial_Elliptic of from_color * to_color * x * y * aspect_ratio
      (** Radial gradient with elliptic isocolor curves. *)
;;
(** Gradient modes to fill ellipses. Each mode specifies a gradient
    from [from_color] to [to_color], with the corresponding method.
    See documentation for type [rectangle_gradient_mode]. *)

type arc_gradient_mode =
   | Arc_Horizontal of from_color * to_color
      (** Horizontal gradient from c1 to c2 *)
   | Arc_Vertical of from_color * to_color
      (** Vertical gradient *)
   | Arc_Diagonal1 of from_color * to_color
      (** Gradient along the first bissector of the circonscript square of
        the arc. *)
   | Arc_Diagonal2 of from_color * to_color
      (** Gradient along the second bissector of the circonscript square of
        the arc. *)
   | Arc_Circular of from_color * to_color
      (** Circular gradient *)
;;
(** Gradient modes to fill arcs. Each mode specifies a gradient
    from [from_color] to [to_color], with the corresponding method. *)

val grad_rectangle : rectangle_gradient_mode ->
  x -> y -> Ugraphics.w -> Ugraphics.h -> unit;;
(** Fill the specified rectangle with the given [rectangle_gradient_mode]. *)

val grad_square : rectangle_gradient_mode ->
  x -> y -> Ugraphics.length -> unit;;
(** Fill the specified square with the given [rectangle_gradient_mode]. *)

val grad_ellipse : ellipse_gradient_mode ->
  x -> y -> Ugraphics.rx -> Ugraphics.ry -> unit;;
(** Fill the specified ellipse with the given [ellipse_gradient_mode]. *)

val grad_circle : ellipse_gradient_mode -> x -> y -> Ugraphics.r -> unit;;
(** Fill the specified circle with the given [ellipse_gradient_mode]. *)

val grad_arc : arc_gradient_mode ->
  x -> y -> rx -> ry -> Ugraphics.from_angle -> Ugraphics.to_angle -> unit;;
(** Fill the specified arc with the given [arc_gradient_mode]. *)
