CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: fetch_utils.fpp,v 1.2 2014/01/10 19:22:10 fclement Exp $

C     
C     Routines to fetch data from the low-level memory and write them
C     into regular Fortran values.
C     
      
#include "pio_constants.h"
      
C     
C     Primitives to fetch data from the memory.
C     
      
      subroutine rand_fetch(sbr, io, loc, odatum, idx, len)
C     Random fetching of data from the memory.
C     Fetches at location loc len bytes to write them in odatum (output
C     data to read) starting from index idx.
      
      implicit none
      
      include 'memory_definition.fh'
            
C     Input arguments.
      character sbr*(*)
      integer io, loc, idx, len
      
C     Output argument.
      character odatum*(*)
      
C     Local variables.
      integer end_idx, end_loc
      
      call check_memory_clean(sbr, io, .True.)
      
      end_idx = idx + (len - 1)
      end_loc = loc + (len - 1)
      
      call verify_rand_fetch(sbr, io, loc, end_loc)
      
      odatum(idx:end_idx) = memory(io)(loc:end_loc)
      
      return
      end
      
      
      subroutine seq_fetch(sbr, io, odatum, idx, len)
C     Sequential fetching of data from the memory.
C     Fetches at location fetch_ptr len bytes to write them in odatum
C     (output data to read) starting from idx.
C     The fetch pointer fetch_ptr is incremented to reflect that len
C     bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Output argument.
      character odatum*(*)
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_fetch_ptr
      
      loc = get_fetch_ptr(io)
      
      call verify_seq_fetch_precondition(sbr, io, len)
      call rand_fetch(sbr, io, loc, odatum, idx, len)
      call advance_fetch_ptr(io, len)
      
      return
      end
      
      
C     
C     Random fetching of basic values from the memory.
C     
      
      character function rand_fetch_char(sbr, io, loc)
C     Fetches one byte from the memory at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
C     External function.
      logical pio_get_debug_mode
      
      call rand_fetch(sbr, io, loc, rand_fetch_char, 1, 1)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *)
     +        'Character ''', rand_fetch_char, ''' fetched.'
      endif
      
      return
      end
      
      
      subroutine rand_fetch_string(sbr, io, loc, sdatum, idx, len)
C     Fetches len bytes from the memory starting at location loc and
C     writes them into string sdatum starting at index idx.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, idx, len
      
C     Output argument.
      character sdatum*(*)
      
C     Local variable.
      integer end_idx
      
C     External function.
      logical pio_get_debug_mode
      
      call rand_fetch(sbr, io, loc, sdatum, idx, len)
      
      if (pio_get_debug_mode()) then
         end_idx = idx + (len - 1)
         write(PIO_REPORT, *)
     +        'String "', sdatum(idx:end_idx), '" fetched.'
      endif
      
      return
      end
      
      
      integer function rand_fetch_int(sbr, io, loc)
C     Fetches INT_SIZE bytes from the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
C     Local variable.
      character s*INT_SIZE
      
C     External functions.
      logical pio_get_debug_mode
      integer string_to_int_bin
      
      call rand_fetch(sbr, io, loc, s, 1, INT_SIZE)
      rand_fetch_int = string_to_int_bin(s)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *)
     +        'Integer ', rand_fetch_int, ' fetched.'
      endif
      
      return
      end
      
      
      FLOAT function rand_fetch_float(sbr, io, loc)
C     Fetches FLOAT_SIZE bytes from the memory starting at location loc.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
C     Local variable.
      character s*FLOAT_SIZE
      
C     External functions.
      logical pio_get_debug_mode
      FLOAT string_to_float_bin
      
      call rand_fetch(sbr, io, loc, s, 1, FLOAT_SIZE)
      rand_fetch_float = string_to_float_bin(s)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *)
     +        'Float ', rand_fetch_float, ' fetched.'
      endif
      
      return
      end
      
      
C     
C     Sequential fetching of basic values from the memory.
C     
      
      character function seq_fetch_char(sbr, io)
C     Fetches one byte from the memory at current fetch_ptr location.
C     The fetch pointer fetch_ptr is incremented to reflect that one
C     byte has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     External function.
      logical pio_get_debug_mode
      
      call seq_fetch(sbr, io, seq_fetch_char, 1, 1)
      
      if (pio_get_debug_mode()) then
         write(PIO_REPORT, *)
     +        'Character ''', seq_fetch_char, ''' fetched.'
      endif
      
      return
      end
      
      
      subroutine seq_fetch_string(sbr, io, sdatum, idx, len)
C     Fetches len bytes from the memory starting at current fetch_ptr
C     location and writes them into string sdatum starting at index idx.
C     The fetch pointer fetch_ptr is incremented to reflect that len
C     bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Output argument.
      character sdatum*(*)
      
C     Local variable.
      integer end_idx
      
C     External function.
      logical pio_get_debug_mode
      
      call seq_fetch(sbr, io, sdatum, idx, len)
      
      if (pio_get_debug_mode()) then
         end_idx = idx + (len - 1)
         write(PIO_REPORT, *)
     +        'String "', sdatum(idx:end_idx), '" fetched.'
      endif
      
      return
      end
      
      
      subroutine seq_fetch_ident(sbr, io, sdatum, idx, len)
C     Fetches len bytes from the memory starting at current fetch_ptr
C     location and writes them into ident sdatum starting at index idx.
C     The fetch pointer fetch_ptr is incremented to reflect that len
C     bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Output argument.
      character sdatum*(*)
      
C     Local variable.
      integer end_idx
      
C     External function.
      logical pio_get_debug_mode
      
      call seq_fetch(sbr, io, sdatum, idx, len)
      
      if (pio_get_debug_mode()) then
         end_idx = idx + (len - 1)
         write(PIO_REPORT, *)
     +        'Ident "', sdatum(idx:end_idx), '" fetched.'
      endif
      
      return
      end
      
      
      integer function seq_fetch_int(sbr, io)
C     Fetches INT_SIZE bytes from the memory starting at current
C     fetch_ptr location.
C     The fetch pointer fetch_ptr is incremented to reflect that
C     INT_SIZE bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     Local variable.
      integer loc
      
C     External functions.
      integer get_fetch_ptr
      integer rand_fetch_int
      
      loc = get_fetch_ptr(io)
      
      seq_fetch_int = rand_fetch_int(sbr, io, loc)
      call advance_fetch_ptr(io, INT_SIZE)

      return
      end
      
      
      FLOAT function seq_fetch_float(sbr, io)
C     Fetches FLOAT_SIZE bytes from the memory starting at current
C     fetch_ptr location.
C     The fetch pointer fetch_ptr is incremented to reflect that
C     FLOAT_SIZE bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     Local variable.
      integer loc
      
C     External functions.
      integer get_fetch_ptr
      FLOAT rand_fetch_float
      
      loc = get_fetch_ptr(io)
      
      seq_fetch_float = rand_fetch_float(sbr, io, loc)
      call advance_fetch_ptr(io, FLOAT_SIZE)
      
      return
      end
      
      
C     
C     Fetching of compound values from the memory (tuples, vectors,
C     matrices).
C     
      
      subroutine rand_fetch_string_vect
     +     (sbr, io, loc, svdatum, vlen, idx, len)
C     Fetches from the memory (starting at loc) the subvector of length
C     len starting from index idx in svdatum.
C     More precisely, rand_fetch_string_vect fetches
C     $\sum_{i=idx}^{idx+len-1} vlen(i)$ bytes from the memory starting
C     at location loc and writes them as len string values into the
C     vector svdatum starting at index idx.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Input/output argument.
      integer loc, vlen(*)
      
C     Output argument.
      character svdatum(*)*(*)
      
C     Local variable.
      integer i
      
      do i = idx, idx + len - 1
         call rand_fetch_string(sbr, io, loc, svdatum(i), 1, vlen(i))
         loc = loc + vlen(i)
      enddo
      
      return
      end
      
      
      subroutine rand_fetch_int_vect(sbr, io, loc, ivdatum, idx, len)
C     Fetches from the memory (starting at loc) the subvector of length
C     len starting from index idx in ivdatum.
C     More precisely, rand_fetch_int_vect fetches len*INT_SIZE bytes
C     from the memory starting at location loc and writes them as len
C     integer values into the vector ivdatum starting at index idx. 
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, idx, len
      
C     Output argument.
      integer ivdatum(*)
      
C     Local variable.
      integer i
      
C     External function.
      integer rand_fetch_int
      
      do i = idx, idx + len - 1
         ivdatum(i) = rand_fetch_int(sbr, io, loc + (i-idx)*INT_SIZE)
      enddo
      
      return
      end
      
      
      subroutine rand_fetch_float_vect(sbr, io, loc, fvdatum, idx, len)
C     Fetches from the memory (starting at loc) the subvector of length
C     len starting from index idx in fvdatum.
C     More precisely, rand_fetch_float_vect fetches len*FLOAT_SIZE bytes
C     from the memory starting at location loc and writes them as len
C     real values into the vector fvdatum starting at index idx.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, idx, len
      
C     Output argument.
      FLOAT fvdatum(*)
      
C     Local variable.
      integer i
      
C     External function.
      FLOAT rand_fetch_float
      
      do i = idx, idx + len - 1
         fvdatum(i) =
     +        rand_fetch_float(sbr, io, loc + (i-idx)*FLOAT_SIZE)
      enddo
      
      return
      end
      
      
      subroutine seq_fetch_string_vect(sbr, io, svdatum, vlen, idx, len)
C     
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Input/output argument.
      integer vlen(*)
      
C     Output argument.
      character svdatum(*)*(*)
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_fetch_ptr
      
      loc = get_fetch_ptr(io)
      
      call rand_fetch_string_vect(sbr, io, loc, svdatum, vlen, idx, len)
      call set_fetch_ptr(io, loc)
      
      return
      end
      
      
      subroutine seq_fetch_int_vect(sbr, io, ivdatum, idx, len)
C     Fetches from the memory (starting at current fetch_ptr location)
C     the subvector of length len starting from index idx in ivdatum.
C     More precisely, seq_fetch_int_vect fetches len*INT_SIZE bytes
C     from the memory starting at location fetch_ptr and writes them
C     as len integer values into the vector ivdatum starting at
C     index idx.
C     The fetch pointer fetch_ptr is incremented to reflect that
C     len*INT_SIZE bytes has been fetched.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Output argument.
      integer ivdatum(*)
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_fetch_ptr
      
      loc = get_fetch_ptr(io)
      
      call rand_fetch_int_vect(sbr, io, loc, ivdatum, idx, len)
      call advance_fetch_ptr(io, len*INT_SIZE)
      
      return
      end
      
      
      subroutine seq_fetch_float_vect(sbr, io, fvdatum, idx, len)
C     Fetches from the memory (starting at current fetch_ptr location)
C     the subvector of length len starting from index idx in fvdatum.
C     More precisely, seq_fetch_float_vect fetches len*FLOAT_SIZE bytes
C     from the memory starting at location fetch_ptr and writes them
C     as len real values into the vector fvdatum starting at
C     index idx.
C     The fetch pointer fetch_ptr is incremented to reflect that
C     len*FLOAT_SIZE bytes has been fetched.

      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idx, len
      
C     Output argument.
      FLOAT fvdatum(*)
      
C     Local variable.
      integer loc
      
C     External function.
      integer get_fetch_ptr
      
      loc = get_fetch_ptr(io)
      
      call rand_fetch_float_vect(sbr, io, loc, fvdatum, idx, len)
      call advance_fetch_ptr(io, len*FLOAT_SIZE)
      
      return
      end
